<div class="posting">
    <div class="postheader">
        <h1 class="subject">
            <a href="{{ env.path + entry.permalink }}">{{ entry.title }}</a>
        </h1>
        <span class="date">{{ entry.date.strftime("%d.%m.%Y, %H:%M") }}</span>
    </div>
    <div class="postbody">
        {{ entry.content }}
    </div>
    <div class="postfooter">
        {% if conf.disqus_shortname and env.type == 'index' %}
            <a class="floatright" href="{{ conf.www_root + entry.permalink }}#disqus_thread">Kommentieren</a>
        {% endif %}
        {% if 'tag' in env.views and entry.tags %}
            <p>verschlagwortet als
                {% for link in entry.tags | tagify -%}
                    <a href="{{ env.path + link.href }}">{{ link.title }}</a>
                    {%- if loop.revindex > 2 -%}
                    ,
                    {%- elif loop.revindex == 2 %}
                    und
                    {% endif %}
                {% endfor %}
            </p>
        {% elif not entry.draft %}
            <p>nicht verschlagwortet</p>
        {% endif %}
    </div>
    <div class="comments">
        {%- if conf.disqus_shortname and env.type == 'entry' and not entry.draft %}
        <div id="disqus_thread"></div>
        <script type="text/javascript">
            var disqus_shortname = '{{ conf.disqus_shortname }}'; // required: replace example with your forum shortname

            // The following are highly recommended additional parameters. Remove the slashes in front to use.
            var disqus_identifier = "{{ conf.www_root + entry.permalink }}";
            var disqus_url = "{{ conf.www_root + entry.permalink }}";

            /* * * DON'T EDIT BELOW THIS LINE * * */
            (function() {
                var dsq = document.createElement('script'); dsq.type = 'text/javascript'; dsq.async = true;
                dsq.src = '{{ env.protocol }}://' + disqus_shortname + '.disqus.com/embed.js';
                (document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0]).appendChild(dsq);
            })();
        </script>
        <noscript>
            <p>Please enable JavaScript to view the <a href="{{ env.protocol }}://disqus.com/?ref_noscript">comments powered by Disqus.</a></p>
        </noscript>
        <a href="{{ env.protocol }}://disqus.com" class="dsq-brlink">
            blog comments powered by <span class="logo-disqus">Disqus</span>
        </a>
        {% endif -%}
    </div>
</div>
